/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#ifndef UKUI_MENU_MENU_PROVIDER_H
#define UKUI_MENU_MENU_PROVIDER_H

#include <QtPlugin>
#include <QAction>

#define UKUI_MENU_CONTEXTMENU_TYPE          "UKUI_MENU_CONTEXTMENU"
#define UKUI_MENU_CONTEXTMENU_IFACE_IID     "org.ukui.menu.contextMenu"
#define UKUI_MENU_CONTEXTMENU_IFACE_VERSION "1.0.0"

namespace UkuiMenu {

class MenuInfo
{
    Q_GADGET
public:
    enum Location {
        AppList = 0,
        Extension,
        FolderPage,
        FullScreen,
        FullScreenFolder,
    };
    Q_ENUM(Location)
};

class MenuProvider
{
public:
    enum RequestType {
        DataType = 0,
        Custom,
    };
    virtual ~MenuProvider() = default;
    virtual int index() = 0;
    virtual bool isSupport(const RequestType &type) = 0;
    virtual QList<QAction*> generateActions(QObject *parent, const QVariant &data, const MenuInfo::Location &location, const QString &locationId) = 0;
};

} // UkuiMenu

Q_DECLARE_INTERFACE(UkuiMenu::MenuProvider, UKUI_MENU_CONTEXTMENU_IFACE_IID)

#endif //UKUI_MENU_MENU_PROVIDER_H
